#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 17-Aug-2003
################################################################################
#
# Mail - Script part of Ethereal Realms designed to compose, retreive, view
#        and manage mail
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Login;						# Login functionality
use eRealms::Mail;						# Mail handler
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table creation
use eRealms::Template;						# Template handling


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle
my $tmpl;							# Template handler

my %mail;							# Mail hash


################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling
	$param = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Mail hash
	$database->GetHashMail(\%mail);

	# Template instance
	$tmpl = new eRealms::Template(\%mail);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new eRealms::Login($database, $cgi, $param);

	if ($login->GetVerificationNormal())
	{
		# Decisions
		# Remove mail
		if (defined($param->{$mail{'TxtMailDelete'}}))
		{ MailPurge($database, $cgi, $param, $tmpl, \%mail); }

		# Sender
		elsif (defined($param->{'SEND'}))
		{ MailSender($database, $cgi, $param, $tmpl, \%mail); }
		
		# Viewer
		elsif (defined($param->{'MID'}))
		{ MailViewer($database, $cgi, $param, $tmpl, \%mail); }
		
		# Composer
		elsif (defined($param->{'WRITE'}))
		{ MailComposer($database, $cgi, $param, $tmpl, \%mail); }

		# Mail listing
		else
		{ MailList($database, $cgi, $param, $tmpl, \%mail); }
	}

	# Document footer
	$database->DocumentGetFooter();

#################################################################################
# Sub-Routines
################################################################################

#####################
# MailComposer
#
# Allows for the composition of mail to be sent out to the designated person.
# Since the list of puppets can get long, this will requires a textfield.

sub MailComposer
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template management
	my $mail     = shift;					# Mail hash

	my $table;						# Table handling

	my $body;						# Body of message
	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting

	my $inline;						# Inline parameter
	my $url;						# Self reference

	my $puppets;						# Display of drop down
	my @puppets;						# List of puppets


	#####################
	# Program area

	# Create instance
	$table = new eRealms::Table();

	# Retreive puppet list
	$database->GetListPuppetYours(\@puppets, $param->{'USER'});

	# Determine values
	# Reply
	if (defined($param->{'REPLY'}))
	{
		# Retreive information
		($_,
		 $from, 
		 $rcpt, 
		 $subj, 
		 $time, 
		 $body) = $database->DataGetPuppeteerMail($param->{'REPLY'});

		# Formatted time
		$time = $database->DataGetDate();
		
		# Inverse values
		$_ = $rcpt;
		$rcpt = $from;
		$from = $_;

		# Message message row
		$body =~ s/^(.+)/$rcpt Wrote on $time:\n\n$1\n---\n\n/s;

		# Subject line
		unless ($subj =~ /\(RE\)/)
		{
			# Replace subject line
			$subj =~ s/^(.+)/(RE) $subj/s;
		}
	}

	# Normal message
	else
	{
		# Determine values
		$from = (defined($param->{'MFROM'})) ? $param->{'MFROM'} : $puppets[0];
		$rcpt = (defined($param->{'MRCPT'})) ? $mail->{'TxtMailInvalid'} : '';
		$subj = (defined($param->{'MSUBJ'})) ? $param->{'MSUBJ'} : '';
		$body = (defined($param->{'MBODY'})) ? $param->{'MBODY'} : '';
	}


	# Determine puppets
	$puppets = $cgi->popup_menu('MFROM', \@puppets, $from);


	# Generate back
	$url    = $cgi->url(-absolute=>1);
	$inline = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'});


	# Display title
	$tmpl->ShowMain($mail->{'TagTitleComposer'});

	# Back button
	print "<A HREF=\"$url$inline\">$mail->{TagBack}</A>\n";


	# Display area
	# Start of form, table
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), SEND=>'True'), "\n";

	# Table top
	$table->MakeTop();

	# Header
	$table->MakeSingle( $tmpl->PassTitle($mail->{'TagMailHeader'}));
	$table->MakeValid(
	 $tmpl->PassSub($mail->{'TagMailFrom'}), 
	 $puppets);
	$table->MakeValid(
	 $tmpl->PassSub($mail->{'TagMailTo'}),
	 $cgi->textfield('MRCPT',$rcpt,25,30));
	$table->MakeValid(
	 $tmpl->PassSub($mail->{'TagMailSubject'}),
	 $cgi->textfield('MSUBJ', $subj,25,250));

	# Body
	$table->MakeBlank();
	$table->MakeSingle($tmpl->PassTitle($mail->{'TagMailMessage'}));
	$table->MakeSingle($cgi->textarea(-name=>'MBODY',-rows=>10,-columns=>60,-wrap=>'SOFT',-default=>$body));

	# Submit and clear
	$table->MakeBlank();
	$table->MakeValid($cgi->submit($mail->{'TxtMailSend'}), $cgi->reset($mail->{'TxtMailReset'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# MailList
#
# Shows a listing of all currently received emails, giving an option to delete, view, reply 
# to them in some matter or the other.

sub MailList
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template viewer
	my $mail     = shift;					# Mail hash

	my $statement;						# Query statement

	my $id;							# Identification number;
	my $from;						# Origin of message
	my $subj;						# Subject of message
	my $time;						# Time of posting

	my $color;						# Selected color
	my $count    = 0;					# Arbitrary counter
	my $url      = $cgi->url(-absolute=>1);			# Current location
	my $inline;						# Inline parameter list


	#####################
	# Program area

	# Data gathering
	# Prepare and execute
	$database->Pull(\$statement, "SELECT MailIDNumber,
		 MailFrom,
		 MailSubject,
		 getDate(MailTimestamp)
		FROM Mail
		WHERE PuppeteerLogin=?
		ORDER BY MailTimestamp",

	 $param->{'USER'}
	);


	# Display title
	$tmpl->ShowMain($mail->{'TagTitleListing'});

	# Initial display
	print "<CENTER><TABLE WIDTH=\"500\">\n",
	      "\t<TR BGCOLOR=\"$mail->{SetOptColor}\">\n",
	      "\t\t<TD>$mail->{TagListFrom}</TD>\n",
	      "\t\t<TD>$mail->{TagListSubject}</TD>\n",
	      "\t\t<TD>$mail->{TagListDate}</TD>\n",
	      "\t</TR>\n";

	# Retreive and display
	while (($id, $from, $subj, $time) = $statement->fetchrow())
	{
		# Increment
		$count++;

		# Determine values
		# Generate inline
		$inline = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'}, MID=>$id);

		# Determine colour
		$color = (($count % 2) != 0) ? $mail->{'SetSecColor'} : $mail->{'SetPriColor'};

		# Information display
		# Data display
		print "\t<TR BGCOLOR=\"$color\">\n",
		      "\t\t<TD><B><A HREF=\"$url$inline\">$from</A></B></TD>\n",
		      "\t\t<TD>$subj</TD>\n",
		      "\t\t<TD>$time</TD>\n",
		      "\t</TR>\n";
	}


	# Determine values
	$inline = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'}, WRITE=>'True');

	# Summary display
	print "\t<TR BGCOLOR=\"$mail->{SetOptColor}\">\n",
	      "\t\t<TD>$mail->{'TagListTotal'} $count</TD>\n",
	      "\t\t<TD COLSPAN=\"2\" ALIGN=\"RIGHT\"><A HREF=\"$url$inline\">$mail->{TagLinkCompose}</A></TD>\n",
	      "\t</TR>\n";


	# End of table
	print "</TABLE><CENTER>\n<BR>\n";

	# End execution
	$statement->finish();
}

#####################
# Mail Purge
#
# Allows for the purging of vile messages from the list of known messages.   Simple and easy
# to use.

sub MailPurge
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template listing
	my $mail     = shift;					# Mail hash

	my $id;							# Identification number;


	#####################
	# Program area

	# Retreive id
	$id = $param->{'OID'};

	# Parameter handling
	$param->Cleanup(
		'OID',
		$mail->{'TagMailSent'}
	 );

	
	# Remove entry
	$database->Quick("DELETE FROM Mail 
		WHERE MailIDNumber=?
		  AND PuppeteerLogin=?",

	 $id,
	 $param->{'USER'});

	
	# Call listing
	MailList($database, $cgi, $param, $tmpl, $mail);
}



#####################
# MailSender
#
# Allows for composed message to be sent out to the designated person.

sub MailSender
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template
	my $mail     = shift;					# Mail hash

	my $send;						# Mail handler
	my $table;						# Table handler

	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $body;						# Body of message

	my $id;							# ID Number of message
	my $notice;						# Notice message

	my $email;						# Email address
	my $name;						# Name of target
	my $target;						# Display of drop down

	my %system;						# Systems hash


	#####################
	# Program area

	# Create instance
	$table = new eRealms::Table($cgi);


	# Data retreival
	$from = $param->{'MFROM'};	
	$rcpt = $param->{'MRCPT'};
	$subj = $param->{'MSUBJ'};
	$body = $param->{'MBODY'};

	# Determine puppeteer
	($target) = $database->DataGetPuppeteerLogin($param->{'MRCPT'});


	# Determine action
	# Processing
	if (defined($target))
	{
		# Retreive full name
		($name)  = $database->DataGetPuppeteerName($target);
		($email) = $database->DataGetPuppeteerEmail($target);

		# Pull ID
		($id)     = $database->DataGetInsert('seqMail');

		# Insert into database
		$database->Write("INSERT INTO Mail
			(MailIDNumber,
			 PuppeteerLogin,
			 MailFrom,
			 MailRcpt,
			 MailSubject,
			 MailContents)
			VALUES(?,?,?,?,?,?)", 

		 $id,
		 $target,
		 $from,
		 $rcpt,
		 $subj,
		 $body);


		# Final preperations
		# Retreive values
		($notice) = $database->DocumentGetMailNotice();
				
		# Build link
		my $url    = $cgi->url();
		my $inline = $param->Crypt(MID=>$id, MRCPT=>$rcpt, MFROM=>$from, MSUBJ=>$subj);


		# Compose and send email
		# Link to hash
		$database->GetHashSystem(\%system);

		# Message information
		if (defined($notice))
		{
			# Send mail
			# Initiate instance
			$send = tie(*MAIL, 'eRealms::Mail', $database);

			# Initial setup
			$send->SetSubject("You have mail from: $from");
			$send->SetSearch(
				AUTHSTRING  => "$url$inline",
				FULLNAME    => $name,
				ORIGIN      => $from,
				RETURNEMAIL => $system{'SetInfoContactAddress'},
				SERVERNAME  => $system{'SetInfoServer'},
				SUBJECT     => $subj
			 );

			# Recipients
			$send->AddTo("$name <$email>");

			# Message
			print MAIL $notice;

			# Close and Send
			close(MAIL);
		}

		# Show listing
		MailList($database, $cgi, $param, $tmpl, $mail);

		# Display success
		$table->MakeTop();
		$table->MakeSingle($mail->{'TagMailSent'});
		$table->MakeBottom();

	}

	# Imporper puppet name
	else
	{
		# Recall composer
		MailComposer($database, $cgi, $param, $tmpl, $mail);
	}
}


#####################
# MailViewer
#
# Allows for the viewing of mail to be sent out to the designated person. This
# will allow the puppeteer to reply if necessary as to make life simpler.

sub MailViewer
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $tmpl     = shift;					# Template instance
	my $mail     = shift;					# Mail hash

	my $table;						# Table handler

	my $body;						# Body of message
	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $user;						# Intended target

	my $inline;						# Inline parameter
	my $url;						# Self reference

	my $id;							# Identification number;


	#####################
	# Program area

	# Create instance
	$table    = new eRealms::Table();

	# Identification number
	$id = $param->{'MID'};
	$param->Cleanup('MID');

	# Assign OID
	$param->{'OID'} = $id;

	# Retreive information
	($user,
	 $from, 
	 $rcpt, 
	 $subj, 
	 $time, 
	 $body) = $database->DataGetPuppeteerMail($id);

	# Message message row
	$body =~ s/\n/<BR>\n/gm;


	# Only complete if intended target
	if ($user =~ /^$param->{'USER'}/i)
	{
		# Generate back
		$url    = $cgi->url(-absolute=>1);
		$inline = $param->EmbedInline(USER=>$param->{'USER'}, CRYPT=>$param->{'CRYPT'});


		# Display title
		$tmpl->ShowMain($mail->{'TagTitleViewer'});

		# Back button
		print "<A HREF=\"$url$inline\">$mail->{TagBack}</A>\n";


		# Display area
		# Start of form
		print $cgi->start_form();
		print $param->EmbedNormal($param->Flat(), REPLY=>$id, WRITE=>'True'), "\n";

		# Table top
		$table->MakeTop();

		# Header
		$table->MakeSingle($tmpl->PassTitle($mail->{'TagMailHeader'}));
		$table->MakeValid(
		 $tmpl->PassSub($mail->{'TagMailFrom'}),
		 $from);
		$table->MakeValid(
		 $tmpl->PassSub($mail->{'TagMailTo'}),
		 $rcpt);
		$table->MakeValid(
		 $tmpl->PassSub($mail->{'TagMailSubject'}),
		 $subj);
		$table->MakeValid(
		 $tmpl->PassSub($mail->{'TagMailDate'}),
		 $time);
			
		# Body
		$table->MakeBlank();
		$table->MakeSingle($tmpl->PassTitle($mail->{'TagMailMessage'}));
		$table->MakeSingle($body);

		# Submit and remove
		$table->MakeBlank();
		$table->MakeValid($cgi->submit($mail->{'TxtMailReply'}), $cgi->submit($mail->{'TxtMailDelete'}));
		$table->MakeBottom();

		# Form bottom
		print $cgi->end_form(), "\n";
	}

	# Invalid component
	else
	{
		# Call listing
		MailList($database, $cgi, $param, $tmpl, $mail);
	}
}
